@@ -269,6 +269,12 @@ public interface NinaAction {
 	 */
 	public void setAccept(boolean accept);
 
+	/**
+	 * 
+	 * @param skipSpace
+	 */
+	public void setSkipSpaces(boolean skipSpace);
+
 	/**
 	 * 
 	 * @param q
