@@ -40,6 +40,12 @@ public interface NinaLanguageOption {
 	 */
 	public String getLookaheadMark();
 
+	/**
+	 * 
+	 * @return
+	 */
+	public String getLookaheadMarkInit();
+
 	/**
 	 * 
 	 * @return
@@ -66,6 +72,24 @@ public interface NinaLanguageOption {
 	 */
 	public String getAppendBuffer(String s);
 
+	/**
+	 * 
+	 * @param s
+	 * @param ch
+	 * @return
+	 */
+	public String getAppendBuffer(String s, int ch);
+
+	/**
+	 * 
+	 * @param s
+	 * @param bufferVariable
+	 * @param radix
+	 * @return
+	 */
+	public String getAppendBuffer(String s, String bufferVariable,
+			int radix);
+
 	/**
 	 * 
 	 * @return
@@ -90,6 +114,13 @@ public interface NinaLanguageOption {
 	 */
 	public String getSequenceBufferName();
 
+	/**
+	 * 
+	 * @param suffix
+	 * @return
+	 */
+	public String getSequenceBufferName(String suffix);
+
 	/**
 	 * 
 	 * @param radix
@@ -97,6 +128,14 @@ public interface NinaLanguageOption {
 	 */
 	public String getBufferToInteger(int radix);
 
+	/**
+	 * 
+	 * @param bufferVariable
+	 * @param radix
+	 * @return
+	 */
+	public String getBufferToInteger(String bufferVariable, int radix);
+
 	/**
 	 * 
 	 * @param radix
