@@ -1359,6 +1359,49 @@ public class NinaMatcherTest extends TestCase {
 		mat(m, "aaa aaa aaa");
 		nmt(m, "");
 		nmt(m, "aaaa ");
+		nmt(m, "Aaa aAa aaA");
+	}
+
+	public void test0213() {
+		String s =
+				"#machine NFABuilder\n" +
+				"#option disableTurnBySlash\n" +
+				" =======                                @*****\n" +
+				" =aaa  >------------/(?i)[a-z]+/-------->aa  *\n" +
+				" ==^====                                **v***\n" +
+				"   +----------------/ /-------------------+  \n" +
+				"";
+		NinaPattern m;
+
+		m = Nina.pattern("A", s, Collections.<String, String>emptyMap(),
+				new NinaSubautomata("a"),
+				new NinaConsoleStub());
+		mat(m, "aBCa");
+		mat(m, "aaCa aAa");
+		mat(m, "Aaa aAa aaA");
+		nmt(m, "");
+		nmt(m, "aaaa ");
+	}
+
+	public void test0214() {
+		String s =
+				"#machine NFABuilder\n" +
+				"#option disableTurnBySlash\n" +
+				" =======                                @*****\n" +
+				" =aaa  >------------/\\/+/--------------->aa  *\n" +
+				" ==^====                                **v***\n" +
+				"   +----------------/ /-------------------+  \n" +
+				"";
+		NinaPattern m;
+
+		m = Nina.pattern("A", s, Collections.<String, String>emptyMap(),
+				new NinaSubautomata("a"),
+				new NinaConsoleStub());
+		mat(m, "////");
+		mat(m, "//// ///");
+		mat(m, "/// /// ///");
+		nmt(m, "");
+		nmt(m, "//// ");
 	}
 
 }
