@@ -116,6 +116,7 @@ implements NinaTranslator {
 	private Set<Integer> accepts = new HashSet<Integer>();
 	private Set<Integer> deads = new HashSet<Integer>();
 	private Set<Integer> emptyTransitions = new HashSet<Integer>();
+	private Set<Integer> skipSpaces = new HashSet<Integer>();
 	private Stack<DFAState<Object, ?, Void>> trz =
 			new Stack<DFAState<Object, ?, Void>>();
 	private Map<Range, String> rangeNameMap =
@@ -184,6 +185,9 @@ implements NinaTranslator {
 					((DBS)state).isOnlyEmptyTransition()) {
 				emptyTransitions.add(stateNo);
 			}
+			if(state instanceof DBS && ((DBS)state).isSkipSpaces()) {
+				skipSpaces.add(stateNo);
+			}
 			trz.push(state);
 			return stateNo++;
 		}
@@ -275,6 +279,14 @@ implements NinaTranslator {
 		return emptyTransitions;
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	protected Set<Integer> skipSpacesIterable() {
+		return skipSpaces;
+	}
+
 	/**
 	 * 
 	 * @param state
@@ -483,6 +495,12 @@ implements NinaTranslator {
 	 */
 	protected abstract void printRangeMethods(PrintStream out);
 
+	/**
+	 * 
+	 * @param out
+	 */
+	protected abstract void printSkipSpaces(PrintStream out);
+
 	/**
 	 * 
 	 * @param ot
@@ -744,6 +762,9 @@ implements NinaTranslator {
 		env.bind("LINECONT", tmp);
 		env.bind("USE_IMMEDIATE_FN",
 				q.isOptionDefined("useImmediateFunction") ? "t" : "");
+		env.bind("IGNORE_CASE",
+				q.isOptionDefined("ignoreCase") ? "t" : "");
+		env.bind("SKIP_SPACES", q.getOptionNvl("whiteSpaces", " \\t\\n"));
 
 		if("prompt".equals(q.getOption("main"))) {
 			env.bind("PUTPROMPT", "true");
@@ -1122,6 +1143,18 @@ implements NinaTranslator {
 				return 0;
 			}
 
+		});
+		cmd.putCommand("print_skipspaces", new ShProcess() {
+
+			@Override
+			public int main(ShEnvironment env, ShFileSystem fs,
+					InputStream stdin, PrintStream stdout,
+					PrintStream stderr,
+					String... args) throws IOException {
+				getsubfa(args).printSkipSpaces(stdout);
+				return 0;
+			}
+
 		});
 
 		if(o instanceof NFABuilder) {
