@@ -786,6 +786,14 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		printPStates(out, emptyTransitionsIterable());
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printSkipSpaces(java.io.PrintStream)
+	 */
+	@Override
+	public void printSkipSpaces(PrintStream out) {
+		printPStates(out, skipSpacesIterable());
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptToken(java.io.PrintStream)
 	 */
