@@ -16,7 +16,8 @@
 #
 putlicense_cstyle.sh
 
-[ -z "$PACKAGE" ] || echo "package $PACKAGE;"
+[ -z "$PACKAGE" ] || echo "namespace $PACKAGE"
+[ -z "$PACKAGE" ] || echo "{"
 [ -z "$EXTENDS" ] || [ -z "$IMPLEMENTS" ] || cln=':'
 print_imports
 cat definition
@@ -26,6 +27,7 @@ then
   nullsym='-1'
   [ -z "$NEWLINEMODE" ] && streamsym='System.IO.TextReader'
   [ -z "$NEWLINEMODE" ] || streamsym='System.IO.Stream'
+  streamfile='System.IO.FileStream'
   [ -z "$NEWLINEMODE" ] && readfn='Read'
   [ -z "$NEWLINEMODE" ] || readfn='ReadByte'
 elif [ -z "$LTYPE" ]
@@ -34,6 +36,7 @@ then
   nullsyml='-1'
   [ -z "$NEWLINEMODE" ] && streamsym='System.IO.TextReader'
   [ -z "$NEWLINEMODE" ] || streamsym='System.IO.Stream'
+  streamfile='System.IO.FileStream'
   [ -z "$NEWLINEMODE" ] && readfn='Read'
   [ -z "$NEWLINEMODE" ] || readfn='ReadByte'
 else
@@ -48,14 +51,20 @@ cat << EOF
 public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $cln $EXTENDS $IMPLEMENTS
 {
 
+#region parser code
 	/* @@@-PARSER-CODE-START-@@@ */
 	class TokenException : System.Exception
 	{
+		public int LineNumber { get; private set; }
+		public TokenException(int lineNumber) : base("Syntax error in line " + lineNumber)
+		{
+			LineNumber = lineNumber;
+		}
 	}
 
 	interface Engine
 	{
-		int Step($streamsym __rd, $CTYPE c) $tre;
+		int Step($CTYPE c) $tre;
 		bool Accepted();
 		bool Dead();
 		bool EmptyTransition();
@@ -65,6 +74,7 @@ public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $cln $EXTENDS $IMPLEMENTS
 		int DeadState();
 		int StateSize();
 		int FinallyState();
+		bool SkipSpaces();
 	}
 EOF
 
@@ -96,7 +106,6 @@ EOF
 [ -z "$TYPE" ] && echo '	private const int Invalidtoken = 0x7fff7fff;'
 [ -z "$TYPE" ] || echo '	private static readonly object Invalidtoken = new {};'
 cat << EOF
-	readonly ${CLASSNAME}${TEMPLATE} __this__;
 	private const int NinaEOF = -1;
 	private const int NinaAccept = -1;
 	private const int NinaFail = -9;
@@ -116,27 +125,50 @@ cat << EOF
 	private int __slen = 0;
 	private $CTYPE unread = $nullsym;
 
+#pragma warning disable 169
 	$VALTYPE _;
-	System.Collections.Generic.List<$VALTYPE> _l;
-	object yieldObject;
-	$streamsym befstream = null;
-	System.Exception exception;
-
 	System.Text.StringBuilder _buffer;
+	System.Text.StringBuilder _bufferUnicode;
 	int _int;
 	object _bigint;
 	object _num;
+#pragma warning restore 169
 
-	void _initlist() {
+	System.Collections.Generic.List<$VALTYPE> _l;
+	object yieldObject;
+
+#pragma warning disable 414
+	System.Exception exception;
+#pragma warning restore 414
+
+	System.Collections.Generic.List<$streamsym> streamStack =
+			new System.Collections.Generic.List<$streamsym>();
+
+	int lineNumber = 1;
+	System.Collections.Generic.List<int> lineNumberStack =
+			new System.Collections.Generic.List<int>();
+
+	bool yieldIsEof = false;
+
+	void _initlist()
+	{
 		_l = new System.Collections.Generic.List<$VALTYPE>();
 	}
 
-	void _addlist($VALTYPE o) {
+	void _addlist($VALTYPE o)
+	{
 		_l.Add(o);
 	}
 
 EOF
 
+[ -z "$TYPE" ] && cat << EOF
+	bool _isWhiteSpace(int ch) {
+		return "$SKIP_SPACES".IndexOf((char)ch) >= 0;
+	}
+
+EOF
+
 [ -n "$ENABLE_INDENT" ] && cat << EOF
 	private int _indent;
 	private int _indentBefore;
@@ -158,7 +190,6 @@ do
   ch1=`replace_strange_char $i`
   echo "		ENGINE_${ch1} = new _Engine_${ch1}(this);"
 done
-echo "		__this__ = this;"
 echo "	}"
 
 [ -n "$ENABLE_LOG" ] && cat << EOF
@@ -168,6 +199,7 @@ if [ -n "$TYPE" ]; then
 elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
   echo '	private int __unreadcomment = -2;'
 fi
+[ -n "$COMMENT" ] && echo '	bool COMMENT = true;'
 
 if [ -n "$ENABLE_BACKTRACK" ]
 then
@@ -198,20 +230,123 @@ fi
 cat field
 [ -n "$LEXER" ] && echo "	private $LCTYPE _unreadl = $nullsyml;"
 [ -n "$LEXER" ] || echo "	private $CTYPE _unreadl = $nullsym;"
+[ -n "$LINECONT" ] && [ -z "$TYPE" ] && echo "	private int _unreadc = $nullsym;"
 echo
 
+function _putcom {
+echo
+echo "	int ReadComment()"
+echo "	{"
+if [ -n "$LINECONT" ]; then
+  cat << EOF
+		int c, c2;
+		if(_unreadc >= 0)
+		{
+			c = _unreadc;
+			_unreadc = -1;
+		}
+		else
+		{
+			c = ReadComment0();
+		}
+		if(c == ${sq}${LINECONT}${sq})
+		{
+			c2 = ReadComment0();
+			if(c2 == '\n')
+			{
+				return ReadComment();
+			}
+			else if(c2 == '\r')
+			{
+				_unreadc = ReadComment0();
+				_unreadc = _unreadc == '\n' ? -1 : _unreadc;
+				return ReadComment();
+			}
+			else
+			{
+				_unreadc = c2;
+				return c;
+			}
+		}
+		else
+		{
+			return c;
+		}
+EOF
+else
+  echo '		int c = ReadComment0();'
+  echo "		return c;"
+fi
+echo '	}'
+}
+
 function _putreadf {
-  if [ -n "$3" ]; then
-  elif [ "$COMMENT" = "shell" ]; then
-    cat << EOF
-	int ReadComment($streamsym stream)
+  echo
+  echo "	void INCLUDE($streamsym rd) {"
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		if(__lookahead_ptr >= 0) {'
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '			throw new System.Exception();'
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		}'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '		if(__backtrack_ptr >= 0) {'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '			throw new System.Exception();'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '		}'
+#  [ -n "$ENABLE_INDENT"    ] && echo '		if(__indent_unread >= 0) {'
+#  [ -n "$ENABLE_INDENT"    ] && echo '			throw new System.Exception();'
+#  [ -n "$ENABLE_INDENT"    ] && echo '		}'
+  echo "		streamStack.Add(rd);"
+  echo "		lineNumberStack.Add(lineNumber);"
+  echo "		lineNumber = 1;"
+  echo "	}"
+  echo
+  if [ -n "$streamfile" ]; then
+    echo "	void INCLUDE(string name) {"
+    echo '		System.IO.Stream ins;'
+    echo
+    echo "		ins = new ${streamfile}(name,"
+    echo "				System.IO.FileMode.Open,"
+    echo "				System.IO.FileAccess.Read);"
+    echo "		INCLUDE(new System.IO.StreamReader(ins));"
+    echo '	}'
+    echo
+  fi
+  cat << EOF
+	int ReadChar()
 	{
 		int c;
 
-		c = stream.${readfn}();
-		if(c == ${sq}${COMMENTHEADER}${sq})
+		c = streamStack[streamStack.Count - 1].${readfn}();
+		if(c == '\n')
 		{
-			while((c = stream.${readfn}()) >= 0)
+			lineNumber++;
+		}
+		return c;
+	}
+EOF
+  [ -n "$CUSTOMREADER" ] || cat << EOF
+	int ReadFromStream() $tre {
+		return ReadChar();
+	}
+EOF
+
+  if [ -n "$IGNORE_CASE" ]; then
+    ignorecase='System.Char.ToUpper((char)c)'
+  else
+    ignorecase='c'
+  fi
+  if [ -n "$3" ]; then
+  elif [ "$COMMENT" = "shell" ]; then
+    cat << EOF
+	int ReadComment0()
+	{
+		int c;
+
+		c = ReadFromStream();
+		if(!COMMENT)
+		{
+			return c;
+		}
+		else if(c == ${sq}${COMMENTHEADER}${sq})
+		{
+			while((c = ReadFromStream()) >= 0)
 			{
 				if(c == '\n')
 				{
@@ -222,13 +357,14 @@ function _putreadf {
 		}
 		else
 		{
-			return c;
+			return $ignorecase;
 		}
 	}
 EOF
+  _putcom
   elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
     cat << EOF
-	int ReadComment($streamsym stream)
+	int ReadComment0()
 	{
 		int c, state = 0;
 
@@ -236,11 +372,15 @@ EOF
 		{
 			c = __unreadcomment;
 			__unreadcomment = -2;
-			return c;
+			return $ignorecase;
 		}
-		else if((c = stream.${readfn}()) == '/')
+		else if(!COMMENT)
 		{
-			while((c = stream.${readfn}()) >= 0)
+			return ReadFromStream();
+		}
+		else if((c = ReadFromStream()) == '/')
+		{
+			while((c = ReadFromStream()) >= 0)
 			{
 				switch(state)
 				{
@@ -270,7 +410,7 @@ EOF
 					{
 						return ' ';
 					}
-					else
+					else if(c != '*')
 					{
 						state = 100;
 					}
@@ -288,30 +428,55 @@ EOF
 		}
 		else
 		{
-			return c;
+			return $ignorecase;
 		}
 	}
 EOF
+  _putcom
   else
     cat << EOF
-	int ReadComment($streamsym stream)
+	int ReadComment0()
 	{
-		return stream.${readfn}();
+		int c = ReadFromStream();
+		return $ignorecase;
 	}
 EOF
+  _putcom
   fi
 
-  [ -n "$ENABLE_INDENT" ] || echo "	$2 $1($streamsym stream)"
-  [ -n "$ENABLE_INDENT" ] && echo "	$2 ${1}_1($streamsym stream)"
+  [ -n "$ENABLE_INDENT" ] || echo "	$2 $1()"
+  [ -n "$ENABLE_INDENT" ] && echo "	$2 ${1}_1()"
   echo '	{'
+
   if [ -s read ]; then
     cat read
   elif [ -z "$3" ]; then
     cat << EOF
 		int c;
 
-		c = ReadComment(stream);
-		return c < 0 ? NinaEOF : c;
+		while(streamStack.Count > 0)
+		{
+			if((c = ReadComment()) >= 0)
+			{
+				return c;
+			}
+EOF
+    if [ -n "$streamfile" ]; then
+      echo '			else if(streamStack.Count > 1)'
+      echo '			{'
+      echo '				streamStack[streamStack.Count - 1].Close();'
+      echo '				streamStack.RemoveAt(streamStack.Count - 1);'
+      echo '				lineNumber = lineNumberStack[lineNumberStack.Count - 1];'
+      echo '				lineNumberStack.RemoveAt(lineNumberStack.Count - 1);'
+      echo '			}'
+    fi
+    cat << EOF
+			else
+			{
+				streamStack.RemoveAt(streamStack.Count - 1);
+			}
+		}
+		return NinaEOF;
 EOF
   else
     echo "		return ($2)stream.next();"
@@ -320,7 +485,7 @@ EOF
 
   if [ -n "$ENABLE_INDENT" ]; then
     cat << EOF
-	$2 $1($streamsym stream)
+	$2 $1()
 	{
 		int c;
 
@@ -331,12 +496,12 @@ EOF
 		}
 		else if(!__is_line_begin)
 		{
-			c = ${1}_1(stream);
+			c = ${1}_1();
 		}
 		else
 		{
 			_indent = 0;
-			while((c = ${1}_1(stream)) == ' ' || c == '\t')
+			while((c = ${1}_1()) == ' ' || c == '\t')
 			{
 				_indent++;
 			}
@@ -358,7 +523,7 @@ if [ -z "$LEXER" ]; then
   _putreadf _Read1l "$CTYPE" "$TYPE"
   cat << EOF
 
-	$CTYPE _Read1($streamsym stream)
+	$CTYPE _Read1()
 	{
 		$CTYPE c;
 
@@ -370,7 +535,7 @@ if [ -z "$LEXER" ]; then
 EOF
   if [ -z "$TYPE" -a -z "$NEWLINEMODE" ]; then
     cat <<  EOF
-		else if((c = _Read1l(stream)) == '\r' && (c = _Read1l(stream)) != '\n')
+		else if((c = _Read1l()) == '\r' && (c = _Read1l()) != '\n')
 		{
 			_unreadl = c;
 			c = '\r';
@@ -380,7 +545,7 @@ EOF
     cat << EOF
 		else
 		{
-			c = _Read1l(stream);
+			c = _Read1l();
 		}
 EOF
   fi
@@ -393,7 +558,7 @@ else
   nina_template.nfa.cs.sub2.sh "$LEXER" "$LCTYPE" "$LTYPE" "$LEXER" "$LCTYPE"
   cat << EOF
 
-	$LCTYPE _Read1ul($streamsym stream)
+	$LCTYPE _Read1ul()
 	{
 		$LCTYPE c;
 
@@ -405,7 +570,7 @@ else
 EOF
   if [ -z "$LTYPE" -a -z "$NEWLINEMODE" ]; then
     cat <<  EOF
-		else if((c = _Read1l(stream)) == '\r' && (c = _Read1l(stream)) != '\n')
+		else if((c = _Read1l()) == '\r' && (c = _Read1l()) != '\n')
 		{
 			_unreadl = c;
 			c = '\r';
@@ -415,7 +580,7 @@ EOF
     cat << EOF
 		else
 		{
-			c = _Read1l(stream);
+			c = _Read1l();
 		}
 EOF
   fi
@@ -423,7 +588,7 @@ EOF
 		return c;
 	}
 
-	$CTYPE _Read1($streamsym stream)
+	$CTYPE _Read1()
 	{
 		System.Text.StringBuilder b = new System.Text.StringBuilder();
 		object o = null;
@@ -435,7 +600,7 @@ EOF
 		f = ${LEXER}_Accepted();
 		while(o == null)
 		{
-			if((a = _Read1ul(stream)) == $nullsyml)
+			if((a = _Read1ul()) == $nullsyml)
 			{
 				o = ${LEXER}_Gettoken(b);
 				break;
@@ -482,7 +647,7 @@ fi
 
 cat << EOF
 
-	private $CTYPE _Read($streamsym rd) $tre {
+	private $CTYPE _Read() $tre {
 		$CTYPE c;
 
 		while(true)
@@ -518,7 +683,7 @@ EOF
 				{
 					__lookahead = null;
 					__lookahead_ptr = -1;
-					c = _Read(rd);
+					c = _Read();
 				}
 				__Logprint("Read Lookahead: ", c);
 EOF
@@ -534,7 +699,7 @@ EOF
 				{
 					__backtrack = null;
 					__backtrack_ptr = __backtrack_wptr = -1;
-					c = _Read(rd);
+					c = _Read();
 				}
 				__Logprint("Read Backtracking: ", c);
 EOF
@@ -546,7 +711,7 @@ EOF
 				unread = $nullsym;
 EOF
 echo "			}"
-echo "			else if((c = _Read1(rd)) != $nullsym)"
+echo "			else if((c = _Read1()) != $nullsym)"
 echo "			{"
 [ -z "$NEWLINEMODE" ] && echo "				$CTYPE d;"
 [ -n "$ENABLE_BACKTRACK" ] && echo '				__Write_backtrack(c);'
@@ -555,7 +720,7 @@ echo '				__Logprint("Read: ", c);'
 				if(c != '\r')
 				{
 				}
-				else if((d = _Read1(rd)) == '\n')
+				else if((d = _Read1()) == '\n')
 				{
 					c = '\n';
 				}
@@ -870,9 +1035,11 @@ do
   echo "		__this__ = o;"
   echo "	}"
   echo
-  echo "	public int Step($streamsym __rd, $CTYPE __c__)"
+  echo "	public int Step($CTYPE __c__)"
   echo '	{'
+  [ -n "$ENABLE_LOOKAHEAD" ] && echo '#pragma warning disable 219'
   [ -n "$ENABLE_LOOKAHEAD" ] && echo '		bool __l__ = __this__.__lookahead_ok;'
+  [ -n "$ENABLE_LOOKAHEAD" ] && echo '#pragma warning restore 219'
   [ -n "$ENABLE_LOOKAHEAD" ] && echo
   [ -n "$DYNAMICAUTOMATA" ]  && echo '		if((__this__.STATE & NinaDiscardstate) != 0)  return 1;'
   [ -n "$ENABLE_LOOKAHEAD" ] && echo '		__this__.__lookahead_ok = true;'
@@ -956,10 +1123,18 @@ EOF
 			return "${ch1}";
 		}
 
+		public bool SkipSpaces() {
+EOF
+    print_skipspaces $i
+    cat << EOF
+		}
+
 	}
 EOF
 done
 
+print_range_methods
+
 for i in $LRPARSERS
 do
   y.tab.cs.sh $i
@@ -1008,7 +1183,7 @@ c='$c'
 		{
 		}
 
-		public int Step($streamsym __rd, $CTYPE __c__)
+		public int Step($CTYPE __c__)
 		{
 			if(ptr >= a.Length)
 			{
@@ -1112,7 +1287,7 @@ EOF
 ch1=`replace_strange_char ${MAINNAME}`
 echo
 cat << EOF
-	private $CTYPE _Parse($streamsym rd, $CTYPE x, bool? rt, bool skip, ref int st)
+	private $CTYPE _Parse($CTYPE x, bool? rt, bool skip, ref int st)
 	{
 		bool b = false, p = skip;
 		$CTYPE c = x;
@@ -1120,11 +1295,7 @@ cat << EOF
 		int a;
 
 		b = __stk[__slen - 1].Accepted();
-		if(rd == null)
-		{
-			throw new System.Exception("can not recurse");
-		}
-		else if((bool)rt)
+		if((bool)rt)
 		{
 			switch(__stk[__slen - 1].Execaction(NinaBegin))
 			{
@@ -1156,7 +1327,7 @@ cat << EOF
 				{
 					p = false;
 				}
-				else if((a = en.Step(rd, c)) > 0)
+				else if((a = en.Step(c)) > 0)
 				{
 					__Logprint("transit to state " + STATE + ": ", c);
 					b = en.Accepted();
@@ -1222,14 +1393,30 @@ EOF
 				else if(c == $nullsym)
 				{
 EOF
-  [ -n "$MATCHER" ] || echo "					if(!b)  throw new ${TOKENERROR}();"
-  [ -n "$MATCHER" ] && cat << EOF
+  if [ -n "$MATCHER" ]; then
+    cat << EOF
 					if(!b)
 					{
 						st = NinaFail;
 						return $nullsym;
 					}
 EOF
+  else
+    cat << EOF
+					if(!b)
+					{
+						if(yieldIsEof)
+						{
+							st = NinaYield;
+							return $nullsym;
+						}
+						else
+						{
+							throw new ${TOKENERROR}(lineNumber);
+						}
+					}
+EOF
+  fi
   cat << EOF
 					st = NinaAccept;
 					return $nullsym;
@@ -1242,11 +1429,27 @@ EOF
 					st = NinaFail;  return $nullsym;
 				}
 
-				if(__stk[__slen - 1].EmptyTransition()) {
+				if(__stk[__slen - 1].EmptyTransition())
+				{
 					// do nothing
 				}
-				else if(!__stk[__slen - 1].Dead()) {
-					c = _Read(rd);
+				else if(!__stk[__slen - 1].Dead())
+				{
+EOF
+  [ -z "$TYPE" ] && cat << EOF
+					if(__stk[__slen - 1].SkipSpaces())
+					{
+						while(_isWhiteSpace(c = _Read()));
+					}
+					else
+					{
+						c = _Read();
+					}
+EOF
+  [ -z "$TYPE" ] || cat << EOF
+					c = _Read();
+EOF
+  cat << EOF
 				}
 				else if(b)
 				{
@@ -1259,6 +1462,7 @@ EOF
 				{
 					__Logprint("match failed: try lookahead: ", c);
 					LOOKAHEAD_RB();
+					c = _Read();
 					b = en.Accepted();
 EOF
   [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
@@ -1271,7 +1475,7 @@ EOF
 				}
 				else if(c == $nullsym)
 				{
-					if(!b)  throw new ${TOKENERROR}();
+					if(!b)  throw new ${TOKENERROR}(lineNumber);
 					st = NinaAccept;  return $nullsym;
 				}
 				else
@@ -1281,9 +1485,9 @@ EOF
 					st = NinaFail;  return $nullsym;
 				}
 			} while(true);
-		} catch(System.Exception e) {
+		} catch(System.Exception) {
 			UNGET(c);
-			throw e;
+			throw;
 		}
 	}
 
@@ -1318,7 +1522,7 @@ EOF
 		return __stk[__slen - 1].Recover(e);
 	}
 
-	${API_ACCESS_MODIFIER} bool Parse($streamsym rd, Engine entry)
+	${API_ACCESS_MODIFIER} bool Parse(Engine entry)
 	{
 		$CTYPE c = 0;
 		bool? b = false;
@@ -1337,7 +1541,7 @@ EOF
 			{
 				try
 				{
-					if((c = _Parse(rd, c, b, skip, ref a)) != $nullsym)
+					if((c = _Parse(c, b, skip, ref a)) != $nullsym)
 					{
 						skip = false;
 					}
@@ -1347,15 +1551,18 @@ EOF
 						{
 							if((b = Execfinally()) != null)
 							{
-								if(!((bool)b))  throw new ${TOKENERROR}();
+								if(!((bool)b))  throw new ${TOKENERROR}(lineNumber);
 								return (bool)b;
 							}
 
 							if(__slen-- <= 1)
 							{
 EOF
-  [ -n "$MATCHER" ] || echo "								throw new ${TOKENERROR}();"
-  [ -n "$MATCHER" ] && echo '								return false;'
+  if [ -n "$MATCHER" ]; then
+    echo '								return false;'
+  else
+    echo "								throw new ${TOKENERROR}(lineNumber);"
+  fi
   cat << EOF
 							}
 						}
@@ -1392,17 +1599,20 @@ EOF
 				catch(System.Exception e)
 				{
 					exception = e;
-					if(__slen <= 0)  throw e;
+					if(__slen <= 0)  throw;
 					while((STATE = Getrecover(e)) < 0)
 					{
 						if((b = Execfinally()) != null)  break;
-						if(__slen-- <= 1)  throw e;
+						if(__slen-- <= 1)  throw;
 					}
 				}
 				b = true;
 			}
 EOF
-  [ -n "$MATCHER" ] || echo "			if(!((bool)b))  throw new ${TOKENERROR}();"
+  if [ -n "$MATCHER" ]; then
+  else
+    echo "			if(!((bool)b))  throw new ${TOKENERROR}(lineNumber);"
+  fi
   cat << EOF
 			return (bool)b;
 		}
@@ -1414,21 +1624,30 @@ EOF
 
 	${API_ACCESS_MODIFIER} bool Parse($streamsym rd)
 	{
-		return Parse(rd, ENGINE_${ch1});
+		streamStack.Add(rd);
+		return Parse(ENGINE_${ch1});
+	}
+
+	${API_ACCESS_MODIFIER} static bool ParseAll($streamsym rd)
+	{
+		${CLASSNAME} o = new ${CLASSNAME}();
+
+		return o.Parse(rd);
 	}
 
 	${API_ACCESS_MODIFIER} void SetStream($streamsym rd) {
-		if(befstream != null)
+		if(streamStack.Count == 0)
 		{
 			throw new System.Exception();
 		}
-		yieldObject = befstream = rd;
+		yieldObject = rd;
+		streamStack.Add(rd);
 	}
 
 	${API_ACCESS_MODIFIER} object ParseNext() {
 		object o;
 
-		if(befstream == null)
+		if(streamStack.Count == 0)
 		{
 			throw new System.Exception();
 		}
@@ -1436,7 +1655,7 @@ EOF
 		{
 			return null;
 		}
-		else if(Parse(befstream, ENGINE_${ch1}))
+		else if(Parse(ENGINE_${ch1}))
 		{
 			if(yieldObject == null)  throw new System.Exception();
 			o = yieldObject;  yieldObject = null;
@@ -1449,13 +1668,6 @@ EOF
 		}
 	}
 
-	${API_ACCESS_MODIFIER} static bool ParseAll($streamsym rd)
-	{
-		${CLASSNAME} o = new ${CLASSNAME}();
-
-		return o.Parse(rd);
-	}
-
 	static void puts(string s)
 	{
 		System.Console.WriteLine(s);
@@ -1481,7 +1693,9 @@ do
   print_constants $i
 done
 echo '	/* @@@-PARSER-CODE-END-@@@ */'
+echo '#endregion'
 
+print_line_directive
 cat fragment
 #cat fragment | replace_action
 
@@ -1507,3 +1721,4 @@ EOF
 EOF
 
 echo '}'
+[ -z "$PACKAGE" ] || echo "}"
