@@ -63,6 +63,16 @@ public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $extends $implements {
 
 	/* @@@-PARSER-CODE-START-@@@ */
 	static class TokenException extends RuntimeException {
+		private int lineNumber;
+
+		public TokenException(int lineNumber) {
+			super("Syntax error in line " + lineNumber);
+			this.lineNumber = lineNumber;
+		}
+
+		public int getLineNumber() {
+			return lineNumber;
+		}
 	}
 
 	static abstract class Engine {
@@ -76,6 +86,7 @@ public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $extends $implements {
 		abstract int deadState();
 		abstract int stateSize();
 		abstract int finallyState();
+		abstract boolean isSkipSpaces();
 	}
 EOF
 
@@ -140,6 +151,12 @@ cat << EOF
 	java.util.Stack<$streamsym> streamStack =
 			new java.util.Stack<$streamsym>();
 
+	int lineNumber = 1;
+	java.util.Stack<Integer> lineNumberStack =
+			new java.util.Stack<Integer>();
+
+	boolean yieldIsEof = false;
+
 	void _initlist() {
 		_l = new java.util.ArrayList<$VALTYPE>();
 	}
@@ -150,6 +167,13 @@ cat << EOF
 
 EOF
 
+[ -z "$TYPE" ] && cat << EOF
+	boolean _isWhiteSpace(int ch) {
+		return "$SKIP_SPACES".indexOf(ch) >= 0;
+	}
+
+EOF
+
 [ -n "$ENABLE_INDENT" ] && cat << EOF
 	private int $indent;
 	private int $indentBefore;
@@ -173,6 +197,7 @@ if [ -n "$TYPE" ]; then
 elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
   echo '	private int __unreadcomment = -2;'
 fi
+[ -n "$COMMENT" ] && echo '	private boolean COMMENT = true;'
 
 if [ -n "$ENABLE_BACKTRACK" ]
 then
@@ -204,20 +229,58 @@ echo
 cat field
 [ -n "$LEXER" ] && echo "	private $LCTYPE _unreadl = $nullsyml;"
 [ -n "$LEXER" ] || echo "	private $CTYPE _unreadl = $nullsym;"
+[ -n "$LINECONT" ] && [ -z "$TYPE" ] && echo "	private int _unreadc = $nullsym;"
+
+function _putcom {
+echo
+echo "	int readComment() $tre {"
+if [ -n "$LINECONT" ]; then
+  cat << EOF
+		int c, c2;
+		if(_unreadc >= 0) {
+			c = _unreadc;
+			_unreadc = -1;
+		} else {
+			c = readComment0();
+		}
+		if(c == ${sq}${LINECONT}${sq}) {
+			c2 = readComment0();
+			if(c2 == '\n') {
+				return readComment();
+			} else if(c2 == '\r') {
+				_unreadc = readComment0();
+				_unreadc = _unreadc == '\n' ? -1 : _unreadc;
+				return readComment();
+			} else {
+				_unreadc = c2;
+				return c;
+			}
+		} else {
+			return c;
+		}
+EOF
+else
+  echo '		int c = readComment0();'
+  echo "		return c;"
+fi
+echo '	}'
+}
 
 function _putreadf {
   echo
   echo "	void INCLUDE($streamsym rd) {"
-  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		if(__lookahead_ptr >= 0) {'
-  [ -n "$ENABLE_LOOKAHEAD" ] && echo '			throw new IllegalStateException();'
-  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		}'
-  [ -n "$ENABLE_BACKTRACK" ] && echo '		if(__backtrack_ptr >= 0) {'
-  [ -n "$ENABLE_BACKTRACK" ] && echo '			throw new IllegalStateException();'
-  [ -n "$ENABLE_BACKTRACK" ] && echo '		}'
-  [ -n "$ENABLE_INDENT"    ] && echo '		if(__indent_unread >= 0) {'
-  [ -n "$ENABLE_INDENT"    ] && echo '			throw new IllegalStateException();'
-  [ -n "$ENABLE_INDENT"    ] && echo '		}'
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		if(__lookahead_ptr >= 0) {'
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '			throw new IllegalStateException();'
+#  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		}'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '		if(__backtrack_ptr >= 0) {'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '			throw new IllegalStateException();'
+#  [ -n "$ENABLE_BACKTRACK" ] && echo '		}'
+#  [ -n "$ENABLE_INDENT"    ] && echo '		if(__indent_unread >= 0) {'
+#  [ -n "$ENABLE_INDENT"    ] && echo '			throw new IllegalStateException();'
+#  [ -n "$ENABLE_INDENT"    ] && echo '		}'
   echo "		streamStack.push(rd);"
+  echo "		lineNumberStack.push(new Integer(lineNumber));"
+  echo "		lineNumber = 1;"
   echo "	}"
   echo
   if [ -n "$streamfile" ]; then
@@ -230,37 +293,63 @@ function _putreadf {
     echo '	}'
     echo
   fi
-
-  if [ -n "$3" ]; then
-  elif [ "$COMMENT" = "shell" ]; then
-    cat << EOF
-	int readComment() $tre {
+  cat << EOF
+	int readChar() $tre {
 		int c;
 
 		c = streamStack.peek().read();
-		if(c == ${sq}${COMMENTHEADER}${sq}) {
-			while((c = streamStack.peek().read()) >= 0) {
+		if(c == '\n') {
+			lineNumber++;
+		}
+		return c;
+	}
+EOF
+  [ -n "$CUSTOMREADER" ] || cat << EOF
+	int readFromStream() $tre {
+		return readChar();
+	}
+EOF
+
+  if [ -n "$IGNORE_CASE" ]; then
+    ignorecase='Character.toUpperCase(c)'
+  else
+    ignorecase='c'
+  fi
+  if [ -n "$3" ]; then
+  elif [ "$COMMENT" = "shell" ]; then
+    cat << EOF
+	int readComment0() $tre {
+		int c;
+
+		c = readFromStream();
+		if(!COMMENT) {
+			return c;
+		} else if(c == ${sq}${COMMENTHEADER}${sq}) {
+			while((c = readFromStream()) >= 0) {
 				if(c == '\n') {
 					break;
 				}
 			}
 			return c;
 		} else {
-			return c;
+			return $ignorecase;
 		}
 	}
 EOF
+  _putcom
   elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
     cat << EOF
-	int readComment() $tre {
+	int readComment0() $tre {
 		int c, state = 0;
 
 		if(__unreadcomment >= -1) {
 			c = __unreadcomment;
 			__unreadcomment = -2;
-			return c;
-		} else if((c = streamStack.peek().read()) == '/') {
-			while((c = streamStack.peek().read()) >= 0) {
+			return $ignorecase;
+		} else if(!COMMENT) {
+			return readFromStream();
+		} else if((c = readFromStream()) == '/') {
+			while((c = readFromStream()) >= 0) {
 				switch(state) {
 				case 0:
 					if(c == '*') {
@@ -280,7 +369,7 @@ EOF
 				case 101:
 					if(c == '/') {
 						return ' ';
-					} else {
+					} else if(c != '*') {
 						state = 100;
 					}
 					break;
@@ -294,16 +383,19 @@ EOF
 			__unreadcomment = -1;
 			return '/';
 		} else {
-			return c;
+			return $ignorecase;
 		}
 	}
 EOF
+  _putcom
   else
     cat << EOF
-	int readComment() $tre {
-		return streamStack.peek().read();
+	int readComment0() $tre {
+		int c = readFromStream();
+		return $ignorecase;
 	}
 EOF
+  _putcom
   fi
 
   [ -n "$ENABLE_INDENT" ] || echo "	$2 $1() $tre {"
@@ -318,8 +410,11 @@ EOF
 			if((c = readComment()) >= 0) {
 				return c;
 EOF
-    [ -n "$streamfile" ] && echo '			} else if(streamStack.size() > 1) {'
-    [ -n "$streamfile" ] && echo '				streamStack.pop().close();'
+    if [ -n "$streamfile" ]; then
+      echo '			} else if(streamStack.size() > 1) {'
+      echo '				streamStack.pop().close();'
+      echo '				lineNumber = lineNumberStack.pop().intValue();'
+    fi
     cat << EOF
 			} else {
 				streamStack.pop();
@@ -912,10 +1007,18 @@ EOF
 			return "${ch1}";
 		}
 
+		boolean isSkipSpaces() {
+EOF
+    print_skipspaces $i
+    cat << EOF
+		}
+
 	};
 EOF
 done
 
+print_range_methods
+
 for i in $LRPARSERS
 do
   y.tab.java.sh $i
@@ -1128,13 +1231,25 @@ EOF
   cat << EOF
 				} else if(c == $nullsym) {
 EOF
-  [ -n "$MATCHER" ] || echo "					if(!b)  throw new ${TOKENERROR}();"
-  [ -n "$MATCHER" ] && cat << EOF
+  if [ -n "$MATCHER" ]; then
+    cat << EOF
 					if(!b) {
 						st[0] = NINA_FAIL;
 						return $nullsym;
 					}
 EOF
+  else
+    cat << EOF
+					if(!b) {
+						if(yieldIsEof) {
+							st[0] = NINA_YIELD;
+							return $nullsym;
+						} else {
+							throw new TokenException(lineNumber);
+						}
+					}
+EOF
+  fi
   cat << EOF
 					st[0] = NINA_ACCEPT;
 					return $nullsym;
@@ -1148,7 +1263,18 @@ EOF
 				if(__stk[__slen - 1].isEmptyTransition()) {
 					// do nothing
 				} else if(!__stk[__slen - 1].isDead()) {
+EOF
+  [ -z "$TYPE" ] && cat << EOF
+					if(__stk[__slen - 1].isSkipSpaces()) {
+						while(_isWhiteSpace(c = _read()));
+					} else {
+						c = _read();
+					}
+EOF
+  [ -z "$TYPE" ] || cat << EOF
 					c = _read();
+EOF
+  cat << EOF
 				} else if(b) {
 					__logprint("accept " + __stk[__slen - 1]);
 					st[0] = NINA_ACCEPT;  return $nullsym;
@@ -1157,6 +1283,7 @@ EOF
 				} else if(__lookaheadw_ptr >= 0) {
 					__logprint("match failed: try lookahead: ", c);
 					LOOKAHEAD_RB();
+					c = _read();
 					b = en.accepted();
 EOF
   [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
@@ -1224,8 +1351,11 @@ EOF
 							if((b = execfinally()) != null)  break ot;
 							if(__slen-- <= 1) {
 EOF
-  [ -n "$MATCHER" ] || echo '								throw new TokenException();'
-  [ -n "$MATCHER" ] && echo '								return false;'
+  if [ -n "$MATCHER" ]; then
+    echo '								return false;'
+  else
+    echo '								throw new TokenException(lineNumber);'
+  fi
   cat << EOF
 							}
 						}
@@ -1260,7 +1390,10 @@ EOF
 				b = Boolean.TRUE;
 			}
 EOF
-  [ -n "$MATCHER" ] || echo '			if(!b.booleanValue())  throw new TokenException();'
+  if [ -n "$MATCHER" ]; then
+  else
+    echo '			if(!b.booleanValue())  throw new TokenException(lineNumber);'
+  fi
   cat << EOF
 			return b.booleanValue();
 		} finally {
@@ -1344,7 +1477,7 @@ EOF
 			try {
 				parseAll(new java.io.StringReader(s));
 			} catch(TokenException e) {
-				System.out.println("syntax error");
+				System.out.println(e.getMessage());
 			}
 		}
 	}
